/**
 * 
 */
package de.mcs.utils.caches;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

/**
 * @author wklaa_000
 *
 */
public class TimedCache<T extends TimedCacheEntry> {

  private final Map<String, T> localCache = new HashMap<>();
  private final Object ThreadLock = new Object();
  private Thread thread;
  private int count;

  /**
   * 
   */
  public TimedCache() {
    count = 0;
  }

  public void put(T entry) {
    synchronized (localCache) {
      localCache.put(entry.getKey(), entry);
    }
  }

  public boolean hasEntry(String key) {
    return localCache.containsKey(key);
  }

  public T get(String key) {
    return localCache.get(key);
  }

  public void removeOldEntries(long cacheInvalidationTime) {
    List<String> toDelete = new ArrayList<>();
    synchronized (localCache) {
      for (Entry<String, T> entry : localCache.entrySet()) {
        if ((entry.getValue().getTime() + cacheInvalidationTime) < System.currentTimeMillis()) {
          toDelete.add(entry.getKey());
        }
      }
      for (String key : toDelete) {
        localCache.remove(key);
      }
    }
  }

  public void asyncRemoveEntries(final long cacheInvalidationTime) {
    if ((thread != null) && thread.isAlive()) {
      return;
    }
    synchronized (ThreadLock) {
      if ((thread != null) && thread.isAlive()) {
        return;
      }
      count++;
      thread = new Thread(new Runnable() {

        @Override
        public void run() {
          try {
            removeOldEntries(cacheInvalidationTime);
          } catch (Exception e) {
            e.printStackTrace();
          }
        }
      });
      thread.start();
    }
  }

  /**
   * 
   */
  public void clear() {
    localCache.clear();
  }

  /**
   * @return
   */
  public int size() {
    return localCache.size();
  }

  /**
   * @return the count
   */
  protected int getCount() {
    return count;
  }

}
